@@ -1,10 +1,24 @@
 #' @export
 py_to_r.lamindb.models.record.Registry <- function(x) {
+  wrap_registry(x)
+}
+
+#' @export
+# nolint start: object_length_linter
+py_to_r.lamindb.models.sqlrecord.Registry <- function(x) {
+  # nolint end: object_length_linter
+  wrap_registry(x)
+}
+
+wrap_registry <- function(py_registry) {
   wrap_python_callable(
-    x,
+    py_registry,
     public = list(
-      from_df = if ("from_df" %in% names(x)) {
-        wrap_with_py_arguments(registry_from_df, x$from_df)
+      from_df = if ("from_df" %in% names(py_registry)) {
+        wrap_with_py_arguments(registry_from_df, py_registry$from_df)
+      },
+      from_dataframe = if ("from_dataframe" %in% names(py_registry)) {
+        wrap_with_py_arguments(registry_from_df, py_registry$from_dataframe)
       }
     ) |>
       purrr::compact()
@@ -31,6 +45,8 @@ registry_from_df <- function(self, ...) {
     )
   }
 
+  args$df <- standardise_list_columns(args$df)
+
   py_object <- unwrap_python(self)
   unwrap_args_and_call(py_object$from_df, args)
 }
