@@ -3,9 +3,7 @@ py_to_r.lamindb.models.transform.Transform <- function(x) { # nolint object_leng
   wrap_python(
     x,
     public = list(
-      view_lineage = function(with_children = TRUE) {
-        view_lineage()
-      }
+      view_lineage = wrap_with_py_arguments(view_lineage_graph, x$view_lineage)
     )
   )
 }
